package uicc.contactmanager;

/**
 * <p>
 *   A <code>FieldView</code> instance represents a field in a contact managed by 
 *   the contact manager.
 * </p>
 * <p>
 *   A <code>FieldView</code> instance may successively represent multiple fields
 *   throughout its lifetime.
 *   <br>
 *   It is good practice, for an application that needs to manipulate 
 *   fields, to create necessary <code>FieldView</code> instances at installation, 
 *   that will be re-used throughout the application's lifetime.
 * </p>
 * <p>
 *   When it is created, a <code>FieldView</code> instance is in DESELECTED state:
 *   <ul>
 *     <li>it is not related to any field in any contact</li>
 *     <li>its instance attributes are not significant and any change to them is useless.</li>
 *   </ul>
 * </p>
 * <p>
 *   Successful invocation of the methods <code>ContactView.getField()</code> and
 *   <code>ContactView.nextField()</code> result in binding the instance to a specific field 
 *   in the contact:
 *   <ul>
 *     <li>all changes to the <code>FieldView</code> are then reflected to the bound contact field;</li>
 *     <li>if the <code>ContactView</code> instance is in the SELECTED state, the 
 *         <code>FieldView</code> instance represents a field in this contact;</li>
 *     <li>if the <code>ContactView</code> instance is in the SELECTED state, any changes to the 
 *         <code>FieldView</code> instance will be stored to the contact;</li>
 *     <li>if the <code>ContactView</code> instance is in the DESELECTED state, any information updated 
 *         into the <code>FieldView</code> instance will be stored into the contact when the <code>ContactView</code>
 *         instance is added to a contacts list.</li>
 *   </ul>
 * </p>
 * <p>
 *   A field is always associated to a field descriptor in the field descriptors list of the 
 *   containing contact manager.
 * </p>
 * <p>
 *   The <code>FieldView</code> class provides methods to retrieve and update the field's unformatted or 
 *   formatted value (binary, Phone number, name or Address).
 * </p>
 * <p>
 *   Methods to read and update field value binary contents can be invoked iteratively for 
 *   smaller parts of the entire field:
 *   <ul>
 *     <li>Each successive call reads or updates the byte(s) that immediately follow the last byte read
 *         or updated.</li>
 *     <li>Read and update operations maintain independent read and update positions.</li>
 *     <li>These positions are set to the beginning of the field value each time a field is selected.</li>
 *     <li>To reset the positions for the same field, simply select the same field again.</li>
 *   </ul>
 * </p>
 */
public interface FieldView {

    /**
     * Field type specifying an address for this contact.
     */
    public final static short TYPE_ADDR = 0x1;
    /**
     * Field type specifying an address for this contact. 
     */
    public final static short TYPE_FORMATED_ADDR = 0xF;
    /**
     * Field type for the birthday of the contact. 
     */
    public final static short TYPE_BDAY = 0x2;
    /**
     * Field type represents a formatted version of a name for the contact entry.
     */
    public final static short TYPE_FORMATTED_NAME = 0x3;
    /**
     * Field type specifying the name for this contact. 
     */
    public final static short TYPE_NAME = 0x4;
    /**
     *  Field type for a voice telephone number.
     */
    public final static short TYPE_TEL = 0x5;
    /**
     *  Field type specifying a photo for a Contact.
     */
    public final static short TYPE_PHOTO = 0x6;
    /**
     * Field type specifying a photo url of a Contact.
     * SCE: is it usefull since we have URL?
     */
    public final static short TYPE_PHOTO_URL = 0x7;
    /**
     * Field type for an e-mail address.
     */
    public final static short TYPE_EMAIL = 0x8;
    /**
     * Field type specifying supplemental information or a comment associated with a Contact.
     */
    public final static short TYPE_NOTE = 0x09;
    /**
     * Field type specifying the last modification date and time of a Contact item.
     */
    public final static short TYPE_REVISION = 0x0A;
    /**
     * Field type specifying the job title for a Contact.
     */
    public final static short TYPE_TITLE = 0x0B;
    /**
     * Field type specifying a unique ID for a Contact.
     */
    public final static short TYPE_UID = 0x0C;
    /**
     * Field type specifying the uniform resource locator for a Contact.
     */
    public final static short TYPE_URL = 0x0D;
    /**
     * Field type specifying a contact role
     */
    public final static short TYPE_ROLE = 0x0E;
    /**
     * Attribute specifying a domestic address
     */
    public static final int ATTR_DOM = 0x1;
    /**
     * Attribute specifying an international address 
     */
    public static final int ATTR_INTL = 0x2;
    /**
     * Attribute specifying a postal address
     */
    public static final int ATTR_POSTAL = 0x4;
    /**
     * Attribute specifying a parcel address
     */
    public static final int ATTR_PARCEL = 0x8;
    /**
     * Attribute specifying a home address or phone number
     */
    public static final int ATTR_HOME = 0x10;
    /**
     * Attribute specifying a work address or phone number
     */
    public static final int ATTR_WORK = 0x20;
    /**
     * Attribute specifying a preferred phone number
     */
    public static final int ATTR_PREF = 0x40;
    /**
     * Attribute specifying a voice phone number
     */
    public static final int ATTR_VOICE = 0x80;
    /**
     * Attribute specifying a fax phone number
     */
    public static final int ATTR_FAX = 0x100;
    /**
     * Attribute specifying a messaging service phone number
     */
    public static final int ATTR_MSG = 0x200;
    /**
     * Attribute specifying a cell phone number
     */
    public static final int ATTR_CELL = 0x400;
    /**
     * Attribute specifying a pager phone number
     */
    public static final int ATTR_PAGER = 0x800;
    /**
     * Attribute specifying a BBS phone number
     */
    public static final int ATTR_BBS = 0x1000;
    /**
     * Attribute specifying a modem phone number
     */
    public static final int ATTR_MODEM = 0x2000;
    /**
     * Attribute specifying a car phone number
     */
    public static final int ATTR_CAR = 0x4000;
    /**
     * Attribute specifying a ISDN phone number
     */
    public static final int ATTR_ISDN = 0x8000;
    /**
     * Attribute specifying a video phone number
     */
    public static final int ATTR_VIDEO = 0x10000;
    /**
     * Attribute specifying a internet mail address
     */
    public static final int ATTR_INTERNET = 0x1;
    /**
     * Attribute specifying a GIF image format
     */
    public static final int ATTR_GIF = 0x2;
    /**
     * Attribute specifying a CGM image format
     */
    public static final int ATTR_CGM = 0x4;
    /**
     * Attribute specifying a WMF image format
     */
    public static final int ATTR_WMF = 0x8;
    /**
     * Attribute specifying a BMP image format
     */
    public static final int ATTR_BMP = 0x10;
    /**
     * Attribute specifying a MET image format
     */
    public static final int ATTR_MET = 0x20;
    /**
     * Attribute specifying a PMB image format
     */
    public static final int ATTR_PMB = 0x40;
    /**
     * Attribute specifying a DIB image format
     */
    public static final int ATTR_DIB = 0x80;
    /**
     * Attribute specifying a PICT image format
     */
    public static final int ATTR_PICT = 0x100;
    /**
     * Attribute specifying a TIFF image format
     */
    public static final int ATTR_TIFF = 0x200;
    /**
     * Attribute specifying a PDF format
     */
    public static final int ATTR_PDF = 0x400;
    /**
     * Attribute specifying a PS format
     */
    public static final int ATTR_PS = 0x800;
    /**
     * Attribute specifying a JPEG image format
     */
    public static final int ATTR_JPEG = 0x1000;
    /**
     * Attribute specifying a MPEG video format
     */
    public static final int ATTR_MPEG = 0x2000;
    /**
     * Attribute specifying a MPEG2 video format
     */
    public static final int ATTR_MPEG2 = 0x4000;
    /**
     * Attribute specifying a AVI video format
     */
    public static final int ATTR_AVI = 0x8000;
    /**
     * Attribute specifying a Quick Time video format
     */
    public static final int ATTR_QTIME = 0x10000;
    /**
     * Attribute specifying a WAVE audio format
     */
    public static final int ATTR_WAVE = 0x1;
    /**
     * Attribute specifying a PCM audio format
     */
    public static final int ATTR_PCM = 0x2;
    /**
     * Attribute specifying a AIFF audio format
     */
    public static final int ATTR_AIFF = 0x4;
    /**
     * Attribute specifying a X509 certificate key
     */
    public static final int ATTR_X509 = 0x1;
    /**
     * Attribute specifying a PGP key
     */
    public static final int ATTR_PGP = 0x2;
    /***************************************
     * Attribute specifying any attributes
     */
    public static final int ATTR_ANY = 0;
    /****************************
     * For formatted address
     */
    /**
     * Key for the sub-field in a formatted address, containing  the country of a particular address.
     */
    public static final short ADDR_COUNTRY = 0x1;
    /**
     * Key for the sub-field in a formatted address, containing  any extra info of a particular address. 
     */
    public static final short ADDR_EXTRA = 0x02;
    /**
     * Key for the sub-field in a formatted address, containing  the locality (for example, a city) of a particular address. 
     */
    public static final short ADDR_LOCALITY = 0x03;
    /**
     * Key for the sub-field in a formatted address, containing  the post office box of a particular address. 
     */
    public static final short ADDR_POBOX = 0x4;
    /**
     * Key for the sub-field in a formatted address, containing  the postal code (for example, a zip code) of a particular address. 
     */
    public static final short ADDR_POSTALCODE = 0x5;
    /**
     * Key for the sub-field in a formatted address, containing  the region (for example, a province, state, or territory) of a particular address. 
     */
    public static final short ADDR_REGION = 0x06;
    /**
     * Key for the sub-field in a formatted address, containing  the street information of a particular address. 
     */
    public static final short ADDR_STREET = 0x7;
    /**
     * Key for the sub-field in a formatted name, containing  the family name.
     */
    public static final short NAME_FAMILY = 0x1;
    /**
     * Key for the sub-field in a formatted name, containing  the given name. 
     */
    public static final short NAME_GIVEN = 0x2;
    /**
     * Key for the sub-field in a formatted name, containing  other alternate name or names. 
     */
    public static final short NAME_OTHER = 0x3;
    /**
     * Key for the sub-field in a formatted name, containing  a prefix to a name. 
     */
    public static final short NAME_PREFIX = 0x4;
    /**
     * Key for the sub-field in a formatted name, containing  a suffix to a name. 
     */
    public static final short NAME_SUFFIX = 0x5;

    /**
     * Returns the FieldDescriptor identifier of the field.
     * 
     * @return field descriptor identifier
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in UNBOUND state.</li>
     *   </ul>
     */
    public int getFieldDescriptorIdentifier();

    /**
     * <p>
     *   Returns field attributes.
     * </p>
     * <p>
     *   The field attributes value is an integer interpreted as a bit field, where each bit is set 
     *   when a specific attribute is present.
     * </p>
     * <p>
     *   The meanings and bit values of attributes are defined in <code>ATTR_*</code> constants.
     * </p>
     * 
     * @return field attributes
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *   </ul>
     */
    public int getAttributes();

    /**
     * <p>
     *   Sets additional field attributes to this field.
     * </p>
     * <p>
     *   Additional attributes are used to set specific attributes to a contact field, 
     *   for example:
     *   <ul>
     *     <li>Set a telephone number as the preferred (<code>ATTR_PREF</code>)</li>
     *     <li>For a photo to set the encoding format (<code>ATTR_JPG</code>,<code>ATTR_GIF</code>,..)</li>
     *   </ul>
     * </p>
     * <p>
     *   The additional attributes are provided in an integer value interpreted as a bit field, where
     *   each bits position represents an additional attribute.
     * </p>
     * <p>
     *   Binary operations can be used to combine attributes, for example: 
     *   <code>( ATTR_HOME | ATTR_POSTAL | ATTR_PREF )</code>
     * </p>
     * <p>
     *   Any attempt to set as additional attributes, mandatory attributes defined in the
     *   field descriptor that corresponds to the field, is ignored. 
     * </p>
     * @param additionalAttributes additional field attributes
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *   </ul>
     *
     */
    public void setAdditionalAttributes(int additionalAttributes);

    /**
     * <p>
     *   Returns the field type.
     * </p>
     * 
     * @return field type
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *   </ul>
     */
    public short getType();

    /**
     * <p>
     *   Reads a sequence of bytes from the field value.
     * </p>
     * <p>
     *   The first call to the method since the instance has been bound to
     *   a field reads from the beginning of the field value.
     * </p>
     * <p>
     *   Subsequent calls to the method will read bytes still unread.
     * </p>
     *   After the end of the field's value has been reached, all subsequent
     *   calls do not modify the buffer provided and return 0, until the
     *   instance is bound to another field.
     * </p>
     *  
     * @param buffer buffer to copy the bytes into
     * @param offset stating offset in the buffer
     * @param maxLength maximum length to read
     *  
     * @return number of bytes read, 0 if no more bytes to read or empty field
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *     <li><code>WRONG_FIELD_TYPE</code> if the field is not a binary field.</li>
     *   </ul>
     */
    public short readBinary(byte[] buffer, short offset,
            short maxLength);

    /**
     * <p>
     *   Updates the field value with a sequence of bytes.
     * </p>
     * <p>
     *   The first call to the method since the instance has been bound to
     *   a field erases any value previously defined and updates the field value,
     *   starting with the first byte.
     * </p>
     * <p>
     *   Subsequent calls to the method will append additional data to the field
     *   value.
     * </p>
     * <p>
     *   If appending all bytes provided to the method would overflow the maximum
     *   field size defined by the related field descriptor, the bytes are updated
     *   up to this maximum size only, and the number of bytes actually written is
     *   returned.
     * </p>
     *  
     * @param buffer buffer containing bytes used to update the field
     * @param offset starting offset in the buffer
     * @param length number of bytes to update
     *  
     * @return number of bytes actually written
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *     <li><code>WRONG_FIELD_TYPE</code> if the field is not a binary field.</li>
     *   </ul>
     */
    public short updateBinary(byte[] buffer, short offset, short length);

    /**
     * <p>
     *   Returns the field's size.
     * </p>
     * <p>
     *   This is the size of the value currently stored in the field.
     * </p>
     * 
     * @return current field size in bytes
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *   </ul>
     */
    public int getFieldSize();

    /**
     * <p>
     *   Reads the field as a phone number with the following format:
     *   <ul>
     *     <li>1st byte represents the TON-NPI</li>
     *     <li>following bytes represent the phone number encoded in packed BCD format.</li>
     *   </ul>
     * </p>
     *  
     * @param buffer buffer to copy the value into
     * @param offset stating offset in the buffer
     * @param length maximum length to read
     * 
     * @return number of bytes read, 0 if no more bytes to read or empty field
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *     <li><code>WRONG_FIELD_TYPE</code> if the field is not a phone number.</li>
     *   </ul>
     */
    public short readPhoneNumber(byte[] buffer, short offset,
            short length);

    /**
     * <p>
     *   Updates the field as a phone number with following format:
     *   <ul>
     *     <li>1st byte represents the TON-NPI</li> 
     *     <li>following bytes represent the phone number encoded in BCD format.</li>
     *   </ul> 
     * </p>
     * @param buffer buffer containing the new value
     * @param offset stating offset in the buffer
     * @param length length to update
     * 
     * @return number of bytes updated
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *     <li><code>WRONG_FIELD_TYPE</code> if the field is not a phone number.</li>
     *   </ul>
     */
    public short updatePhoneNumber(byte[] buffer, short offset,
            short length);

    /**
     * <p>
     *   Reads a subfield from a formatted address.
     * </p>
     * <p>
     *   The subfield value is in UTF-8 format.
     * </p>
     * 
     * @param key key that identifies the sub-field in the formatted address:
     *              ADDR_COUNTRY ,ADDR_EXTRA, ADDR_LOCALITY, ADDR_POBOX, ADDR_POSTALCODE, ADDR_REGION, ADDR_STREET
     * @param buffer buffer to copy the subfield value into
     * @param offset starting offset in the buffer
     * @param length maximum length to read
     * 
     * @return length read
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *     <li><code>WRONG_KEY</code> if <code>key</code> refers to an unknown subfield.</li>
     *     <li><code>WRONG_FIELD_TYPE</code> if the field is not a formatted address.</li>
     *   </ul>
     */
    public short readFormattedAddressField(short key, byte[] buffer,
            short offset, short length);

    /**
     * <p>
     *   Updates a subfield of a formatted address.
     * </p>
     * <p>
     *   The subfield value is in UTF-8 format.
     * </p>
     * 
     * @param key key that identifies the sub-field in the formatted address:
     *              ADDR_COUNTRY ,ADDR_EXTRA, ADDR_LOCALITY, ADDR_POBOX, ADDR_POSTALCODE, ADDR_REGION, ADDR_STREET
     * @param buffer buffer containing the new subfield value
     * @param offset starting offset in the buffer
     * @param length number of bytes to update
     * 
     * @return Number of bytes updated
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *     <li><code>WRONG_KEY</code> if <code>key</code> refers to an unknown subfield.</li>
     *     <li><code>WRONG_FIELD_TYPE</code> if the field is not a formatted address.</li>
     *   </ul>
     */
    public short updateFormattedAddressField(short key,
            byte[] buffer, short offset, short length);

    /**
     * <p>
     *   Reads a subfield from a formatted name.
     * </p>
     * <p>
     *   The subfield value is in UTF-8 format.
     * </p>
     * 
     * @param key key that identifies the sub-field in formatted name:
     *              NAME_FAMILY NAME_GIVEN, NAME_OTHER NAME_PREFIX, NAME_SUFFIX
     * @param buffer buffer to copy the subfield value into
     * @param offset starting offset in the buffer
     * @param length maximum length to read
     * 
     * @return length read
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *     <li><code>WRONG_KEY</code> if <code>index</code> refers to an unknown subfield.</li>
     *     <li><code>WRONG_FIELD_TYPE</code> if the field is not a formatted name.</li>
     *   </ul>
     */
    public short readFormattedName(short key, byte[] buffer,
            short offset, short length);

    /**
     * <p>
     *   Updates a subfield of a formatted name.
     * </p>
     * <p>
     *   The subfield value is in UTF-8 format.
     * </p>
     * 
     * @param key key that identifies the sub-field in the formatted name:
     *              NAME_FAMILY NAME_GIVEN, NAME_OTHER NAME_PREFIX, NAME_SUFFIX
     * @param buffer buffer containing the new subfield value
     * @param offset starting offset in the buffer
     * @param length number of bytes to update
     * 
     * @return number of bytes updated
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>FieldView</code> object is in DESELECTED state.</li>
     *     <li><code>WRONG_KEY</code> if <code>index</code> refers to an unknown subfield.</li>
     *     <li><code>WRONG_FIELD_TYPE</code> if the field is not a formatted name.</li>
     *   </ul>
     */
    public short updateFormattedName(short key, byte[] buffer,
            short offset, short length);
}
